from dash import Dash, dcc, html, Input, Output, ctx, callback, State
import dash_bootstrap_components as dbc
import warnings
app = Dash(external_stylesheets=[dbc.themes.BOOTSTRAP])
import importlib
from mentalweather.WeatherRegression import WeatherRegression
warnings.simplefilter(action='ignore', category=FutureWarning)
from pycountry import countries
from geopy.geocoders import Nominatim
geolocator = Nominatim(user_agent="123")
from datetime import date, datetime, timedelta
import pandas as pd
import math
from meteostat import Stations, Daily

geolocator = Nominatim(user_agent="123")
controls = dbc.Card(
    [   
        html.Div(
            [
                "City: ",
                dcc.Input(id='city', value="Sydney", type="text",style={"width":"7rem", "margin":"0.5rem", "text-align":"center"}),
            ]
        ),
        html.Div(
            [
                "Lat.: ", 
                dcc.Input(id="lat", type="number", min=-90, max=90, style={"width":"6rem", "margin":"0.5rem", "text-align":"center"}),
                "Long.: ",
                dcc.Input(id="lon", type="number", min=-180, max=180, style={"width":"6rem", "margin":"0.5rem", "text-align":"center"}),
            ],

        ),
        html.Hr(),
        html.Div(
            [
                html.Button(' Search ',id='button', style={"padding":"0.5rem"}),
                " for the following ",
                dcc.Input(id="search_days", value=3, type="number", min=1, max=10, step=1, style={"width":"3rem", "margin":"0.5rem", "text-align":"center"}),
                " days",
            ],
        ),
    ], 
    body=True, style={"background-color": "transparent"} 
)

controls_1 = dbc.Card(
    [
        html.Div(
            [
                html.H5("Fill the following 4 criteria:"),
            ],
        ),
        html.Hr(),
        html.Div(
            [
                dbc.Label("Average Temperature"),
                html.Br(),
                dcc.Input(id="low_tavg", type="number", value=5, min=-5, max=40, step=0.5, style={"width":"4rem", "margin":"0.5rem", "text-align":"center"}),
                " ~ ", 
                dcc.Input(id="high_tavg", type="number", value=39, min=-5, max=40, step=0.5, style={"width":"4rem", "margin":"0.5rem", "text-align":"center"}),
                "°C",
            ]
        ),
        html.Hr(),
        html.Div(
            [
                dbc.Label("Max difference in temperature"),
                dcc.Input(id="delta_t_limit", value=20, type="number", style={"width":"4rem", "margin":"0.5rem", "text-align":"center"}),
                "°C",
            ],
        ), 
        html.Hr(),
        html.Div(
            [
                dbc.Label("Average Daily Precipition below"),
                dcc.Input(id="precipitation_limit", value=15, type="number", style={"width":"5rem", "margin":"0.5rem","text-align":"center"}),
                "mm",
            ],
        ),      
        html.Hr(),
        html.Div([
                dbc.Label("Snow Depth below"),
                dcc.Input(id="snow_limit", value=2, type="number", style={"width":"5rem", "margin":"0.5rem","text-align":"center"}),
                "mm",
        ]),
    ],
    body=True, style={"background-color": "transparent"}
)
res_map = dbc.Card(
    [html.Div(dcc.Graph(id="map")),], body=True, style={"width":"100%"}
)
res_table = html.Div(dbc.Table(id="table_container"))
#, style={, "background-image":"linear-gradient(to top left, #accdfa, #f7e5ab)"},
app.layout = dbc.Container(
    [
        dcc.ConfirmDialog(
            id='no_station_found',
            message='Sorry... No station nearby provides weather forecast data.',
        ),
        dcc.Loading(
            id="loading_1",
            type="circle", fullscreen=True, style={"backgroundColor": "rgba(255, 255, 224, 0.5)"},
            children=[html.Div(id="loading_output_1")]
        ),
        html.H2("Find Resorts with Weather Forecast Data", style={"padding-top":"2%", "text-align":"center"}),
        html.Hr(),
        #dbc.Row(controls, style={"margin":10}),
        dbc.Row([
        dbc.Col([dbc.Row(res_map),
                 dbc.Row(res_table, style={"text-align":"center"}),
                 ], md=9),
        dbc.Col(
            [
                dbc.Row(controls, style={"margin":"4px"}),
                dbc.Row(controls_1, style={"margin":"4px", "margin-top":"8px"}),
                #dbc.Col(dcc.Graph(id="map"), md=8),
            ], md=3
        ),
        ]
        ),
    ],
    fluid=True, style={"height":"100vh", "font-family":"Lucida Console","font-size":"16px"}
)

@app.callback(
    [
        Output("city", "value"),
        Output("lat", "value"),
        Output("lon", "value"),
        Output("table_container", "children"),
        Output("map", "figure"),
        Output("no_station_found", 'displayed'),
        Output("loading_output_1", "children"),
    ],
    [
        Input('button', 'n_clicks'),
    ],
    [
        State("city", "value"),
        State("search_days", "value"),
        State("low_tavg", "value"),
        State("high_tavg", "value"),
        State("delta_t_limit", "value"),
        State("precipitation_limit", "value"),
        State("snow_limit", "value"),
    ]
)

def fetch_resorts_data(n_clicks, city, search_days, low_tavg, high_tavg, delta_t_limit, precipitation_limit, snow_limit):
    if (n_clicks is not None) and (n_clicks > 0):  
        import pandas as pd
        from geopy.geocoders import Nominatim
        geolocator = Nominatim(user_agent="123")
        location = geolocator.geocode(city)
        print(location)
        if not location:
            exit
        lat = location.latitude
        lon = location.longitude
        print(lat)

        '''
        elif ctx.triggered_id == "lat":
            lat = 45 if lat is None else lat
            city = geolocator.reverse(str(lat)+','+str(lon))
        elif ctx.triggered_id =="lon":
            lon = -78 if lon is None else lon
            city = geolocator.reverse(str(lat)+','+str(lon))
        elif ctx.triggered_id == "search_days":
            search_days = 3 if search_days is None else search_days
        elif ctx.triggered_id == "low_tavg":
            low_tavg = 5 if low_tavg is None else low_tavg
        elif ctx.triggered_id == "high_tavg":
            high_tavg = 5 if high_tavg is None else high_tavg
        elif ctx.triggered_id == "delta_t_limit":
            delta_t_limit = 10 if delta_t_limit is None else delta_t_limit
        elif ctx.triggered_id =="precipitation_limit":
            precipitation_limit = 10 if precipitation_limit is None else precipitation_limit
        elif ctx.triggered_id == "snow_limit":
            snow_limit = 2 if snow_limit is None else snow_limit
        '''
        
        start = datetime.combine(date.today()+timedelta(days=1), datetime.min.time())
        end = datetime.combine(date.today()+timedelta(days=search_days), datetime.min.time())
        stations = Stations().inventory('daily', (start, end))
        avaliable_stations = stations.fetch()
        avaliable_contries = {}
        for val in avaliable_stations['country']:
            country = countries.get(alpha_2=val).name
            if country not in avaliable_contries:
                avaliable_contries[country] = 1
            else:
                avaliable_contries[country] += 1

        geolocator = Nominatim(user_agent="123")
        '''
        print("Number of weather stations with avaliable forecast data in each countries:")
        for key,val in avaliable_contries.items():
            print(key+": "+str(val))
        print()'''
        
        stations_now = stations.nearby(lat, lon, 300000)
        stations_now = stations_now.fetch()
        if stations_now.empty:
            print("Nearby weather stations do not provide forecasts for the next "+str(search_days)+" days.")
            
        id = stations_now.index.tolist()
        name = stations_now['name'].tolist()
        stations_lat = stations_now['latitude'].tolist()
        stations_lon = stations_now['longitude'].tolist()
        good_stations = []
        good_prcp =  []
        good_delta = []
        good_tavg = []
        good_snow = []
        good_stations_lat = []
        good_stations_lon = []

        for i, station in enumerate(id):
            data = Daily(station, start, end).fetch()
            if len(data)<search_days:
                continue
            total_prcp = 0
            total_snow = 0
            tavg = 0
            delta_t = 0
            goodData = True

            cal = 0
            for val in data['prcp']:
                if math.isnan(val):
                    continue
                cal += 1
                total_prcp += val
            if not cal == 0:
                if not total_prcp == 0:
                    total_prcp = total_prcp/cal
            else:
                goodData = False

            cal = 0
            for val in data['snow']:
                cal += 1
                if math.isnan(val):
                    total_snow += 0
                    continue
                total_snow += val
            if not cal == 0:
                total_snow = total_snow / cal

            cal = 0
            for val in data['tavg']:
                if math.isnan(val):
                    continue
                cal += 1
                tavg += val
            if not cal == 0:
                if not tavg == 0:
                    tavg = tavg/cal
            else:
                goodData = False

            cal = 0
            for (val1, val2) in zip(data['tmin'], data['tmax']):
                if math.isnan(val1) or math.isnan(val2):
                    break
                cal += 1 
                delta_t += (val2 - val1)
            if not cal == 0:
                if not delta_t == 0:
                    delta_t = delta_t/cal
            else:
                goodData = False

            if goodData and total_prcp <= precipitation_limit:
                if delta_t <= delta_t_limit:
                    if total_snow <= snow_limit:
                        if (tavg <= high_tavg) and (tavg >= low_tavg):
                            good_prcp.append(total_prcp)
                            good_snow.append(total_snow)
                            good_delta.append(delta_t)
                            good_tavg.append(tavg)
                            good_stations.append(name[i])
                            good_stations_lat.append(stations_lat[i])
                            good_stations_lon.append(stations_lon[i])
            
        if not good_stations:
            print("No nearby weather stations satisfy the criteria.")
            no_station_found = True
            import pandas as pd
            import plotly.express as px
            px.set_mapbox_access_token('pk.eyJ1IjoidGhlb3NlcnlhdCIsImEiOiJjbHltcjd6OW0wNDNjMmtzNzdyODBrMjc0In0.b2oDRgrEUur1VqMNVSDPfA')
            #px.set_mapbox_access_token(open(".mapbox_token").read())

            df_map = pd.DataFrame()
            df_map['location'] = [city]
            df_map['lat'] = [lat]
            df_map['lon'] = [lon]
            df_map['description'] = ['your location']
            df_map['size'] = [6]
            '''
            MinX, MaxX = min(good_stations_lon), max(good_stations_lon)
            minY, maxY = min(good_stations_lat), max(good_stations_lat)
            MinY = minY - 0.05*(maxY - minY)
            MaxY = maxY + 0.05*(maxY - minY) '''

            map = px.scatter_mapbox(df_map, lat="lat", lon="lon", hover_name="location",
                                    size="size", hover_data={"size": False})
            map.update_layout(margin={"r":0,"t":0,"l":0,"b":0})
            #map.update_layout(mapbox_bounds={"west": minX, "east": maxX, "south": MinY, "north":MaxY})
            df_map = df_map.drop(columns='size')
            print(df_map)
            lat = ("{:.1f}".format(lat))
            lon = ("{:.1f}".format(lon))
            return city, lat, lon, dbc.Table.from_dataframe(df_map), map, no_station_found, ' '

        else:    
            no_station_found = False
            print("Nearby weather stations with less than "+str(precipitation_limit)+"mm\nof total precipitation in the next "+str(search_days)+" days:")
            for station in good_stations:
                print(station)

            import plotly.express as px
            #import plotly.graph_objects as go
            px.set_mapbox_access_token('pk.eyJ1IjoidGhlb3NlcnlhdCIsImEiOiJjbHltcjd6OW0wNDNjMmtzNzdyODBrMjc0In0.b2oDRgrEUur1VqMNVSDPfA')
            #px.set_mapbox_access_token(open(".mapbox_token").read())

            df_map = pd.DataFrame()
            temp_list = location_formater(good_stations, good_stations_lat, good_stations_lon)
            df_map['lat'] = good_stations_lat
            df_map['lon'] = good_stations_lon
            df_map['location'] = temp_list
            df_map['prcp'] = good_prcp
            df_map['tavg'] = good_tavg
            df_map['delta_t'] = good_delta
            df_map['snow'] = good_snow
            df_map['size'] = [6]*df_map.shape[0]

            import pandas as pd
            filepath = 'informations/informations_2.csv'
            #df = pd.read_csv(filepath)
            test = WeatherRegression(filepath, 'Incidence', ['tavg', 'prcp', 'snow'])
            
            incidence_list = []
            happy_list = []
            for i in range(df_map.shape[0]):
                inc_pred, inc_avg, stk_score = test.stacking_app([df_map.loc[i, ['tavg', 'prcp', 'snow']]])
                incidence_list.append(float(inc_pred[0]))
                if float(inc_pred[0]) < inc_avg:
                    happy_list.append(':)')
                else:
                    happy_list.append(':(')
            '''
            MinX, MaxX = min(good_stations_lon), max(good_stations_lon)
            minY, maxY = min(good_stations_lat), max(good_stations_lat)
            MinY = minY - 0.05*(maxY - minY)
            MaxY = maxY + 0.05*(maxY - minY) '''
            round_df_map = df_map[['location', 'lon', 'lat', 'prcp', 'tavg', 'delta_t', 'snow', 'size']].round(decimals=2)
            round_df_map['mental_disease'] = incidence_list
            round_df_map['Happy?'] = happy_list
            round_df_map = round_df_map.round(decimals=4)
            map = px.scatter_mapbox(round_df_map, lat="lat", lon="lon", hover_name="location",
                                    size="size", hover_data={"size": False}, 
                                    color_continuous_scale=px.colors.sequential.Viridis_r,
                                    color="mental_disease", zoom=5)
            map.update_layout(margin={"r":0,"t":0,"l":0,"b":0})
            #map.update_layout(mapbox_bounds={"west": minX, "east": maxX, "south": MinY, "north":MaxY})
            round_df_map = round_df_map.drop(columns='size')
            print(round_df_map)
            lat = ("{:.1f}".format(lat))
            lon = ("{:.1f}".format(lon))
            return city, lat, lon, dbc.Table.from_dataframe(round_df_map), map, no_station_found, ' '

def location_formater(good_stations, good_stations_lat, good_stations_lon):
    from geopy.geocoders import Nominatim
    geolocator = Nominatim(user_agent="123")
    good_cities = []
    for i in range(len(good_stations)):
        location = geolocator.reverse(str(good_stations_lat[i])+','+str(good_stations_lon[i]))
        if location is None:
            good_cities.append(good_stations[i])
            continue

        good_addr = location.raw['address']   
        addr = good_addr.get('city','') + ', ' + good_addr.get('suburb', '')
        if addr.endswith(' ') or addr.startswith(','):
            addr = addr.strip(', ')
            addr = addr.strip(',')
        if len(addr) <= 1:
            addr = good_stations[i]
        good_cities.append(addr)
        
    return good_cities

if __name__ == "__main__":
    #app.run_server(debug=True)
    app.run_server(debug=False,dev_tools_ui=False,dev_tools_props_check=False)


def plot_map(file_path, map_dropdown, start, end):
    # Enter the start and end year (2012~2021) 
    # to get the mental disease incidence rate world map
    from dash import Dash, html, dcc, Input, Output, State, callback
    import dash_bootstrap_components as dbc
    import plotly.express as px
    import pandas as pd
    from pycountry import countries
    df = pd.read_csv(file_path)
    country_code_not_in_pycountry = {"Micronesia (Federated States of)":'FSM',
                                        "Taiwan":'TWN',
                                        "Democratic People's Republic of Korea":'PRK',
                                        "Republic of Korea":'KOR', "Republic of Moldova":'MDA',
                                        "United States of America":'USA', "Bolivia (Plurinational State of)":'BOL',
                                        "Venezuela (Bolivarian Republic of)":'VEN', "Iran (Islamic Republic of)":'IRN',
                                        "Turkey":'TUR', "Palestine":'PSE', "Democratic Republic of the Congo":'COD',
                                        "United Republic of Tanzania":'TZA', "United States Virgin Islands":'VIR'}

    feature = map_dropdown
    df_a = pd.DataFrame(columns = ['Country', 'Year', feature])

    cal = 0
    dict = df.to_dict(orient='index')
    for i in range(df.shape[0]):
        if (df.loc[i, 'Year'].item() > int(start)) and (df.loc[i, 'Year'].item() <= int(end)):
            df_a.loc[cal] = [df.loc[i, 'Country'], df.loc[i, 'Year'].item(), df.loc[i, feature].item()]
            cal+=1

    for i in range(df_a.shape[0]):
        if countries.get(name=df_a.loc[i, 'Country']):
            df_a.loc[i, 'iso_alpha'] = countries.get(name=df_a.loc[i, 'Country']).alpha_3
        else:
            df_a.loc[i, 'iso_alpha'] = country_code_not_in_pycountry[df_a.loc[i, 'Country']]

    fig = px.choropleth(df_a, locations="iso_alpha",
                        color=feature, # lifeExp is a column of gapminder
                        hover_name="Country", # column to add to hover information
                        color_continuous_scale=px.colors.sequential.Viridis_r)
    return fig


def check_box_selection(wg):
    import ipywidgets as widgets
    from ipywidgets import Checkbox, VBox, HBox
    checkboxes = [widgets.Checkbox(value=False, description=label) for label in wg]
    output = widgets.HBox(children=checkboxes)
    display(output) # it is for jupyter notebook to import and display
    return checkboxes

def checked_list(checked_box):
    selected_data = []
    for i in range(0, len(checked_box)):
        #print(checked_box[i])
        if checked_box[i].value == True:
            selected_data = selected_data+[checked_box[i].description]
    return selected_data

def plot_pairplot(filepath, selected_data):
    import pandas as pd
    import matplotlib.pyplot as plt
    import seaborn as sns

    df = pd.read_csv(filepath)
    if len(selected_data) > 0:
        list_pp = ['Incidence'] + selected_data
        #print("Features: ",list_pp)
        sns.pairplot(df[list_pp], 
                    markers='+',
                    plot_kws=dict(linewidth=1))
        plt.show()


"""
Description
-----------
    I made the dataset for EDA with this .py file. 
    The disease data are combined with the meteostat weather data from 2012 to 2022.(2011 ~ 2021 for analysis with tsun data).
    Note: To save time and energy, please Don't run it again because the data are already saved in the file 'informations_2.csv'
          If you were to run it, it takes hours!
          Also, because of the slightly difference in dataset for each year,
          this file does not gaurantee the use for year outside the range of [2012, 2022].

Parameters
----------
Adjust these 2 variables at the start of this file (line 26 & 27):

start_year: int
    the year from which I started to fetch data from meteostat and combine the mental disease data to dataframe.
    In the range of [2012, 2022] because I only requested the data from 2012 to 2022.
end_year: int
    the last year for fetching data from meteostat and combine the mental disease data to dataframe.
    In the range of [start_year, 2022] because I only requested the data from 2012 to 2022.

Returns
-------
The weather and disease data for each year will be saved in the folder 'informations'(will be created if it doesn't exisits)
"""
start_year = 2012
end_year = 2022

from pycountry import countries
from datetime import datetime
import pandas as pd
import numpy as np
import math
import os
import meteostat
from meteostat import Stations, Daily
from multiprocessing import Pool
from concurrent.futures import ThreadPoolExecutor
import time
perf_time0 = time.time()

def annual_average(avg_station, stations_amount, data, feature):
    if not math.isnan(data_start[feature].iat[0]):
        avg_this_station = 0
        avail_days = 0
        goodData = True

        for val in data[feature]:
            if math.isnan(val):
                goodData = False
                break
            avg_this_station += val
            avail_days += 1
            
        if goodData:
            avg_this_station = avg_this_station / avail_days
            avg_station = (avg_station * stations_amount + avg_this_station)/(stations_amount + 1)
            stations_amount += 1
       
    return avg_station, stations_amount
    

def annual_total(total_station, stations_amount, data, feature):
    if not math.isnan(data_start[feature].iat[0]):
        total_this_station = 0
        avail_days = 0
        goodData = True

        for val in data[feature]:
            if math.isnan(val):
                goodData = False
                break
            total_this_station += val
            avail_days += 1
            
        if goodData:
            total_station = (total_station * stations_amount + total_this_station)/(stations_amount + 1)
            stations_amount += 1
       
    return total_station, stations_amount
    


for k in range(start_year, end_year):
    print(k)
    # read the disease data
    prevalence=pd.read_csv("prevalence/global_prevalence_"+str(k)+".csv")
    incidence=pd.read_csv("incidence/global_incidence_"+str(k)+".csv")
    
    for j in range(len(incidence)):
        if 'Taiwan' in incidence.loc[j, 'location_name']:
            incidence.loc[j, 'location_name'] = 'Taiwan'
            break
    for i in range(len(prevalence)):
        if 'Taiwan' in prevalence.loc[i, 'location_name']:
            prevalence.loc[i, 'location_name'] = 'Taiwan'
            break

    informations = {'Country':[], 'Year':[], 'tavg':[], 'delta_t':[], 'snow':[], 'wspd':[], 
                    'wpgt':[], 'pres':[], 'prcp':[], 'Prevalence':[], 'Incidence':[]}
    country_code_not_in_pycountry = {"Micronesia (Federated States of)":'FM',
                                     "Taiwan":'TW',
                                     "Democratic People's Republic of Korea":'KP',
                                     "Republic of Korea":'KR', "Republic of Moldova":'MD',
                                     "United States of America":'US', "Bolivia (Plurinational State of)":'BO', "Venezuela (Bolivarian Republic of)":'VE', "Iran (Islamic Republic of)":'IR', "Turkey":'TR', "Palestine":'PS', "Democratic Republic of the Congo":'CD', "United Republic of Tanzania":'TZ', "United States Virgin Islands":'VI'}
    
    for i in range(len(prevalence)):
        print(i)
        country = prevalence.loc[i,'location_name']
        year = int(prevalence.loc[i,'year'])
        start = datetime(year, 1, 1)
        end = datetime(year, 12, 31)
        if countries.get(name=country):
            stations = Stations().region(countries.get(name=country).alpha_2)
        else:
            stations = Stations().region(country_code_not_in_pycountry[country])
        stations = stations.inventory('daily', (start, end))
        stations = stations.fetch().index.tolist()

        temp_avg, temp_stations_amount, temp_avg1, temp_stations_amount1 = 0, 0, 0, 0
        annual_tavg, annual_tmin, annual_tmax, annual_delta_t, total_snow, total_prcp = 0, 0, 0, 0, 0, 0
        annual_wspd, annual_wpgt, annual_pres = 0, 0, 0
        tavg_stations_amount, tmin_stations_amount, tmax_stations_amount, delta_t_stations_amount = 0, 0, 0, 0
        snow_stations_amount, wspd_stations_amount, wpgt_stations_amount, pres_stations_amount, prcp_stations_amount = 0, 0, 0, 0, 0

        for j, station in enumerate(stations):
            data_start = Daily(station, start, start).fetch()
            # continue if data unavailable
            if data_start.empty:
                continue

            data = Daily(station, start, end).fetch()

            # since the disease data are provide per country per year,
            # we use the average of following weather data for each year

            # calculate tavg of the country
            annual_tavg, tavg_stations_amount = annual_average(annual_tavg, tavg_stations_amount, data, 'tavg')
            # annual_tavg = temp_avg
            # tavg_stations_amount = temp_stations_amount

            # calculate delta_temperature from tmax and tmin of the country
            temp_avg, temp_stations_amount = annual_average(annual_tmax, tmax_stations_amount, data, 'tmax')
            temp_avg1, temp_stations_amount1 = annual_average(annual_tmin, tmin_stations_amount, data, 'tmin')
            if (temp_stations_amount > tmax_stations_amount) and (temp_stations_amount1 > tmin_stations_amount):
                delta_t_stations_amount += 1
                
                annual_delta_t = (annual_delta_t*(delta_t_stations_amount-1) + (temp_avg - temp_avg1)) / delta_t_stations_amount
            
            annual_tmax = temp_avg
            tmax_stations_amount = temp_stations_amount
            annual_tmin = temp_avg1
            tmin_stations_amount = temp_stations_amount1

            # calculate total snow of the country
            temp_avg, temp_stations_amount = annual_average(total_snow, snow_stations_amount, data, 'snow')
            total_snow = temp_avg
            snow_stations_amount = temp_stations_amount

            # calculate wspd(average wind speed) of the country
            temp_avg, temp_stations_amount = annual_average(annual_wspd, wspd_stations_amount, data, 'wspd')
            annual_wspd = temp_avg
            wspd_stations_amount = temp_stations_amount

            # calculate wpgt(peak wind gust) of the country
            temp_avg, temp_stations_amount = annual_average(annual_wpgt, wpgt_stations_amount, data, 'wpgt')
            annual_wpgt = temp_avg
            wpgt_stations_amount = temp_stations_amount

            # calculate pres(average sea-level air pressure in hPa) of the country
            temp_avg, temp_stations_amount = annual_average(annual_pres, pres_stations_amount, data, 'pres')
            annual_pres = temp_avg
            pres_stations_amount = temp_stations_amount

            # calculate total prcp of the country
            temp_avg, temp_stations_amount = annual_total(total_prcp, prcp_stations_amount, data, 'prcp')
            total_prcp = temp_avg
            prcp_stations_amount = temp_stations_amount        
        

        informations['Country'].append(country)
        informations['Year'].append(year)
        if tavg_stations_amount:
            informations['tavg'].append(annual_tavg)
        else:
            informations['tavg'].append(np.nan)

        if delta_t_stations_amount:
            informations['delta_t'].append(annual_delta_t)
        else:
            informations['delta_t'].append(np.nan)

        if snow_stations_amount:
            informations['snow'].append(total_snow)
        else:
            informations['snow'].append(np.nan)
            
        if wspd_stations_amount:
            informations['wspd'].append(annual_wspd)
        else:
            informations['wspd'].append(np.nan)

        if wpgt_stations_amount:
            informations['wpgt'].append(annual_wpgt)
        else:
            informations['wpgt'].append(np.nan)

        if pres_stations_amount:
            informations['pres'].append(annual_pres)
        else:
            informations['pres'].append(np.nan)

        if prcp_stations_amount:
            informations['prcp'].append(total_prcp)
        else:
            informations['prcp'].append(np.nan)

        informations['Prevalence'].append(prevalence.loc[i,'val'])
        informations['Incidence'].append(np.nan)

    df = pd.DataFrame(informations)
    for i in range(len(incidence)):
        df.loc[(df['Country'] == incidence.loc[i,'location_name']) & (df['Year'] == int(incidence.loc[i,'year'])),'Incidence'] = incidence.loc[i,'val']
    outdir = 'informations'
    if not os.path.exists(outdir):
        os.mkdir(outdir)
    df.to_csv("informations/information_"+str(k)+".csv", sep=',', index=False, encoding='utf-8')

inform=pd.read_csv("informations/information_2012.csv")
informations = {'Country':[], 'Year':[], 'tavg':[], 'delta_t':[], 'snow':[], 'wspd':[], 
                    'wpgt':[], 'pres':[], 'prcp':[], 'Prevalence':[], 'Incidence':[]}

for i in range(len(inform)):
    informations['Country'].append(inform.loc[i,'Country'])
    informations['Year'].append(inform.loc[i,'Year'])
    informations['tavg'].append(inform.loc[i,'tavg'])
    informations['delta_t'].append(inform.loc[i,'delta_t'])
    informations['snow'].append(inform.loc[i,'snow'])
    informations['wspd'].append(inform.loc[i,'wspd'])
    informations['wpgt'].append(inform.loc[i,'wpgt'])
    informations['pres'].append(inform.loc[i,'pres'])
    informations['prcp'].append(inform.loc[i,'prcp'])
    informations['Prevalence'].append(inform.loc[i,'Prevalence'])
    informations['Incidence'].append(inform.loc[i,'Incidence'])

for k in range(2013,2022):
    inform=pd.read_csv("informations/information_"+str(k)+".csv")
    for i in range(len(inform)):
        j = 0
        while j < len(informations['Country']) and informations['Country'][j] != inform.loc[i,'Country']:
            j += 1
        while j < len(informations['Country']) and informations['Country'][j] == inform.loc[i,'Country']:
            j += 1

        informations['Country'].insert(j,inform.loc[i,'Country'])
        informations['Year'].insert(j,inform.loc[i,'Year'])
        informations['tavg'].insert(j,inform.loc[i,'tavg'])
        informations['delta_t'].insert(j,inform.loc[i,'delta_t'])
        informations['snow'].insert(j,inform.loc[i,'snow'])
        informations['wspd'].insert(j,inform.loc[i,'wspd'])
        informations['wpgt'].insert(j,inform.loc[i,'wpgt'])
        informations['pres'].insert(j,inform.loc[i,'pres'])
        informations['prcp'].insert(j,inform.loc[i,'prcp'])
        informations['Prevalence'].insert(j,inform.loc[i,'Prevalence'])
        informations['Incidence'].insert(j,inform.loc[i,'Incidence'])
    
    df = pd.DataFrame(informations)
    df.to_csv("informations/informations_2.csv", sep=',', index=False, encoding='utf-8')

perf_time1 = time.time()
took_time = perf_time1 - perf_time0
print(f'It took: {took_time}s')


class WeatherRegression: 
    def __init__(self, filepath, disease_data: str, analyze_data):
        import pandas as pd
        self.disease_data = disease_data
        self.df = pd.read_csv(filepath)
        self.analyze_data = analyze_data
    
    def stacking(self, forcast_data):
        import pandas as pd
        from sklearn.model_selection import train_test_split
        from sklearn.metrics import mean_squared_error, mean_absolute_error, r2_score
        from sklearn.ensemble import RandomForestRegressor, ExtraTreesRegressor, GradientBoostingRegressor
        from sklearn.ensemble import StackingRegressor
        import matplotlib.pyplot as plt
        import warnings
        warnings.simplefilter('ignore')

        list_stk = self.analyze_data + [self.disease_data]
        df_stk = self.df[list_stk]
        df_stk = df_stk.dropna()

        x = df_stk[self.analyze_data]
        y = df_stk[self.disease_data]
        disease_average = y.mean()
        #print(f'The dataset\'s average {self.disease_data} is {disease_average}')
        train_x, test_x, train_y, test_y = train_test_split(x, y, test_size=0.1)
        #print(train_x.values)
        rgr1 = RandomForestRegressor(criterion='squared_error', n_estimators=50, random_state=0,
                                     n_jobs=2, max_depth=10, max_features='log2', 
                                     min_samples_split=30, max_leaf_nodes=100, bootstrap=False)
        rgr2 = ExtraTreesRegressor(criterion='squared_error', n_estimators=100, random_state=0,
                                    n_jobs=2, max_depth=10, max_features='log2',
                                    min_samples_split=30, bootstrap=False)
        fe = GradientBoostingRegressor(n_estimators=25, subsample=0.7, min_samples_split=30,
                                       max_features='log2', random_state=200)
        stk = StackingRegressor(estimators=[('rfr', rgr1), ('etr', rgr2)], final_estimator=fe)
        stk = stk.fit(train_x.values, train_y)
        print('stacking regressor score:', stk.score(test_x, test_y))

        pred_y = stk.predict(test_x)
        mse = mean_squared_error(test_y, pred_y)
        mae = mean_absolute_error(test_y, pred_y)
        r2 = r2_score(test_y, pred_y)
        print(f'MSE: {mse}\nMAE: {mae}\nr2 score: {r2}')
        
        predict = stk.predict(forcast_data)
        '''if predict > disease_average:
            print(f'\nSemtimental...\nThe model predicts that\nthe mental disease incidence will be {predict}\nis higher than dataset\'s average {disease_average}...')
        else:
            print(f'\nHappy :)\nThe model predicts that\nthe mental disease incidence will be {predict}\nis lower than dataset\'s average {disease_average} :)')
        '''
        rgr1 = rgr1.fit(train_x.values, train_y)
        feature_importance_1 = pd.Series(rgr1.feature_importances_, index=train_x.columns).sort_values(ascending=False)
        fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(12,4))
        
        ax1.bar(feature_importance_1.index, feature_importance_1)
        ax1.set_title('RandomForestRegressor')
        ax1.set_ylabel('Feature Importance')

        rgr2 = rgr2.fit(train_x.values, train_y)
        feature_importance_2 = pd.Series(rgr2.feature_importances_, index=train_x.columns).sort_values(ascending=False)
        ax2.bar(feature_importance_2.index, feature_importance_2)
        ax2.set_title('ExtraTreesRegressor')
        ax2.set_ylabel('Feature Importance')
        plt.show()
        return predict
    
    def stacking_app(self, forcast_data):
        import pandas as pd
        from sklearn.model_selection import train_test_split
        from sklearn.metrics import mean_squared_error, mean_absolute_error, r2_score
        from sklearn.ensemble import RandomForestRegressor, ExtraTreesRegressor, GradientBoostingRegressor
        from sklearn.ensemble import StackingRegressor
        import matplotlib.pyplot as plt
        import warnings
        warnings.simplefilter('ignore')

        list_stk = self.analyze_data + [self.disease_data]
        df_stk = self.df[list_stk]
        df_stk = df_stk.dropna()

        x = df_stk[self.analyze_data]
        y = df_stk[self.disease_data]
        disease_average = y.mean()
        #print(f'The dataset\'s average {self.disease_data} is {disease_average}')
        train_x, test_x, train_y, test_y = train_test_split(x, y, test_size=0.1)
        #print(train_x.values)
        rgr1 = RandomForestRegressor(criterion='squared_error', n_estimators=50, random_state=0,
                                     n_jobs=2, max_depth=10, max_features='log2', 
                                     min_samples_split=30, max_leaf_nodes=100, bootstrap=False)
        rgr2 = ExtraTreesRegressor(criterion='squared_error', n_estimators=100, random_state=0,
                                    n_jobs=2, max_depth=10, max_features='log2',
                                    min_samples_split=30, bootstrap=False)
        fe = GradientBoostingRegressor(n_estimators=25, subsample=0.7, min_samples_split=30,
                                       max_features='log2', random_state=200)
        stk = StackingRegressor(estimators=[('rfr', rgr1), ('etr', rgr2)], final_estimator=fe)
        stk = stk.fit(train_x.values, train_y)

        pred_y = stk.predict(test_x)
        mse = mean_squared_error(test_y, pred_y)
        mae = mean_absolute_error(test_y, pred_y)
        r2 = r2_score(test_y, pred_y)
        
        predict = stk.predict(forcast_data)
        
        return predict, disease_average, r2
'''
import pandas as pd
filepath = 'informations_2.csv'
df = pd.read_csv(filepath)
test = WeatherRegression(df, 'Incidence', ['tavg', 'prcp', 'snow', 'pres', 'wspd'], True)
test.stacking([[30, 0, 10]])
'''